@@ -2,10 +2,21 @@
  * connection.h:
  * connection to the pop3 server
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
- * $Id: connection.h,v 1.23 2002/11/10 13:45:47 chris Exp $
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __CONNECTION_H_ /* include guard */
@@ -16,6 +27,8 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 
+#include "poll.h"
+
 #include "authswitch.h"
 #include "buffer.h"
 #include "listener.h"
@@ -89,13 +102,13 @@ struct ioabs {
      * buffering and state issues. post_select should return a combination of
      * the flags defined below, as well as doing any I/O-layer specific
      * handling. */
-    void (*pre_select)(connection c, int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds);
+    void (*pre_select)(connection c, int *n, struct pollfd *pfds);
     
     /* post_select:
      * Do handling after select has completed. Returns 1 if new data have been
      * read, 0 if not. May alter the connection_state of the associated
      * connection. */
-    int (*post_select)(connection c, fd_set *readfds, fd_set *writefds, fd_set *exceptfds);
+    int (*post_select)(connection c, struct pollfd *pfds);
 
     /* shutdown:
      * Shut down the connection. Returns zero on success, IOABS_WOULDBLOCK if
