@@ -2,16 +2,29 @@
  * ioabs_tls.c:
  * I/O abstraction layer for TLS/SSL.
  *
- * Copyright (c) 2002 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2002 Chris Lightfoot.
  * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "configuration.h"
 
 #ifdef USE_TLS
 
-static const char rcsid[] = "$Id: ioabs_tls.c,v 1.14 2003/11/24 19:58:28 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <sys/types.h>
 
@@ -21,7 +34,7 @@ static const char rcsid[] = "$Id: ioabs_tls.c,v 1.14 2003/11/24 19:58:28 chris E
 
 #include <openssl/ssl.h>
 
-#include <sys/select.h>
+#include "poll.h"
 
 #include "connection.h"
 #include "listener.h"
@@ -228,15 +241,16 @@ static ssize_t ioabs_tls_immediate_write(connection c, const void *buf, size_t c
 
 /* ioabs_tls_pre_select:
  * Pre-select handling for TLS, taking account of the rehandshaking nonsense. */
-static void ioabs_tls_pre_select(connection c, int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
+static void ioabs_tls_pre_select(connection c, int *n, struct pollfd *pfds) {
     struct ioabs_tls *io;
     io = (struct ioabs_tls*)c->io;
 
-    FD_SET(c->s, readfds);  /* always want to read */
+    pfds[c->s].fd = c->s;
+    pfds[c->s].events |= POLLIN; /* always want to read */
     if (!io->write_blocked_on_read &&
         (buffer_available(c->wrb) > 0 || io->accept_blocked_on_write
          || io->read_blocked_on_write || io->shutdown_blocked_on_write))
-        FD_SET(c->s, writefds);
+        pfds[c->s].events |= POLLOUT;
 
     if (c->s > *n)
         *n = c->s;
@@ -244,15 +258,15 @@ static void ioabs_tls_pre_select(connection c, int *n, fd_set *readfds, fd_set *
 
 /* ioabs_tls_post_select:
  * Post-select handling for TLS, with its complicated logic. */
-static int ioabs_tls_post_select(connection c, fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
+static int ioabs_tls_post_select(connection c, struct pollfd *pfds) {
     int ret = 0, R = 0;
     ssize_t n, wtotal;
     int canread, canwrite;
     struct ioabs_tls *io;
     io = (struct ioabs_tls*)c->io;
 
-    canread  = FD_ISSET(c->s, readfds);
-    canwrite = FD_ISSET(c->s, writefds);
+    canread  = pfds[c->s].revents & (POLLIN | POLLHUP);
+    canwrite = pfds[c->s].revents & POLLOUT;
     
     /* First, accept handling. */
     if ((io->accept_blocked_on_read && canread) || (io->accept_blocked_on_write && canwrite)) {
@@ -276,14 +290,12 @@ static int ioabs_tls_post_select(connection c, fd_set *readfds, fd_set *writefds
                     return 0;
 
                 case SSL_ERROR_SYSCALL:
-                    if (!e) {
-                        if (ERR_get_error() == 0)
-                            log_print(LOG_ERR, _("ioabs_tls_post_select: client %s: SSL_accept: connection unexpectedly closed by peer"), c->idstr);
-                        else
-                            log_print(LOG_ERR, _("ioabs_tls_post_select: client %s: SSL_accept: %m; closing connection"), c->idstr);
-                        break;
-                    }
-                    /* fall through */
+                    if (ERR_get_error() == 0)
+                        log_print(LOG_ERR, _("ioabs_tls_post_select: client %s: SSL_accept: connection unexpectedly closed by peer"), c->idstr);
+                    else
+                        log_print(LOG_ERR, _("ioabs_tls_post_select: client %s: SSL_accept: %m; closing connection"), c->idstr);
+                    break;
+                        
                 case SSL_ERROR_SSL:
                 default:
                     log_print(LOG_ERR, _("ioabs_tls_post_select: client %s: SSL_accept: %s; closing connection"), c->idstr, ERR_reason_error_string(ERR_get_error()));
