@@ -2,12 +2,25 @@
  * netloop.c:
  * Network event loop for tpop3d.
  *
- * Copyright (c) 2002 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2002 Chris Lightfoot.
  * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-static const char rcsid[] = "$Id: netloop.c,v 1.10 2003/11/24 19:58:28 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -31,6 +44,8 @@ static const char rcsid[] = "$Id: netloop.c,v 1.10 2003/11/24 19:58:28 chris Exp
 #include <sys/socket.h>
 #include <sys/time.h>
 
+#include "poll.h"
+
 #include "config.h"
 #include "connection.h"
 #include "listener.h"
@@ -41,7 +56,7 @@ static const char rcsid[] = "$Id: netloop.c,v 1.10 2003/11/24 19:58:28 chris Exp
 /* The socket send buffer is set to this, so that we don't end up in a
  * position that we send so much data that the client will not have received
  * all of it before we time them out. */
-#define MAX_DATA_IN_FLIGHT      8192
+#define DEFAULT_TCP_SEND_BUFFER     16384
 
 int max_running_children = 16;          /* How many children may exist at once. */
 volatile int num_running_children = 0;  /* How many children are active. */
@@ -105,11 +120,12 @@ static void remove_connection(connection c) {
 
 /* listeners_pre_select:
  * Called before the main select(2) so listening sockets can be polled. */
-static void listeners_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
+static void listeners_pre_select(int *n, struct pollfd *pfds) {
     item *t;
     vector_iterate(listeners, t) {
         int s = ((listener)t->v)->s;
-        FD_SET(s, readfds);
+       pfds[s].fd = s;
+       pfds[s].events |= POLLIN;
         if (s > *n) *n = s;
     }
 }
@@ -117,16 +133,27 @@ static void listeners_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_s
 /* listeners_post_select:
  * Called after the main select(2) to allow listening sockets to sort
  * themselves out. */
-static void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
+static void listeners_post_select(struct pollfd *pfds) {
     item *t;
     vector_iterate(listeners, t) {
         listener L = (listener)t->v;
-        if (FD_ISSET(L->s, readfds)) {
+       if (pfds[L->s].revents & (POLLIN | POLLHUP)) {
             struct sockaddr_in sin, sinlocal;
             size_t l = sizeof(sin);
-            int s, a = MAX_DATA_IN_FLIGHT;
+            static int tcp_send_buf = -1;
+            int s;
             time_t start;
 
+            if (tcp_send_buf == -1) {
+                int q;
+                q = config_get_int("tcp-send-buffer", &tcp_send_buf);
+                if (q <= 0 || tcp_send_buf < 0) {
+                    tcp_send_buf = DEFAULT_TCP_SEND_BUFFER;
+                    if (q == -1 || tcp_send_buf < 0)
+                        log_print(LOG_WARNING, "listeners_post_select: bad value for tcp-send-buffer; using default");
+                }
+            }
+
             time(&start);
             errno = 0;
             
@@ -144,7 +171,8 @@ static void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exc
                     close(s);
                 }
 #endif
-                else if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, &a, sizeof(a)) == -1) {
+                else if (tcp_send_buf != 0
+                         && setsockopt(s, SOL_SOCKET, SO_SNDBUF, &tcp_send_buf, sizeof(tcp_send_buf)) == -1) {
                     /* Set a small send buffer so that we get EAGAIN if the client
                      * isn't acking our data. */
                     log_print(LOG_ERR, "listeners_post_select: setsockopt: %m");
@@ -171,7 +199,7 @@ static void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exc
                 }
             }
 
-            if (errno != EAGAIN)
+            if (errno != EAGAIN && errno != EINTR)
                 log_print(LOG_ERR, "net_loop: accept: %m");
             
         }
@@ -180,12 +208,12 @@ static void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exc
 
 /* connections_pre_select:
  * Called before the main select(2) so connections can be polled. */
-static void connections_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
+static void connections_pre_select(int *n, struct pollfd *pfds) {
     connection *J;
     for (J = connections; J < connections + max_connections; ++J)
         /* Don't add frozen connections to the select masks. */
         if (*J && !connection_isfrozen(*J) && (*J)->cstate != closed)
-            (*J)->io->pre_select(*J, n, readfds, writefds, exceptfds);
+            (*J)->io->pre_select(*J, n, pfds);
 }
 
 /* fork_child CONNECTION
@@ -327,7 +355,7 @@ static int fork_child(connection c) {
             /* Began session. We log a message in a known format, and call
              * into the authentication drivers in case they want to do
              * something with the information for POP-before-SMTP relaying. */
-            log_print(LOG_INFO, _("fork_child: %s: began session for `%s' with %s; child PID is %d"), c->idstr, c->a->user, c->a->auth, (int)ch);
+            log_print(LOG_NOTICE, _("fork_child: %s: began session for `%s' with %s; child PID is %d"), c->idstr, c->a->user, c->a->auth, (int)ch);
             authswitch_onlogin(c->a, c->remote_ip, c->local_ip);
 
             if (childwait) {
@@ -363,31 +391,37 @@ static int fork_child(connection c) {
 /* connections_post_select:
  * Called after the main select(2) to do stuff with connections.
  *
- * For each connection, we call its own post_select routine. This will do all sorts
- * of stuff which is hidden to us, including pushing the running/closing/closed
- * state machine around and reading and writing the I/O buffers. We need to try to
- * parse commands when it's indicated that data have been read, and react to the
- * changed state of any connection. */
-static void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
+ * For each connection, we call its own post_select routine. This will do all
+ * sorts of stuff which is hidden to us, including pushing the
+ * running/closing/closed state machine around and reading and writing the I/O
+ * buffers. We need to try to parse commands when it's indicated that data have
+ * been read, and react to the changed state of any connection. */
+static void connections_post_select(struct pollfd *pfds) {
     static size_t i;
     size_t i0;
     time_t start;
 
     time(&start);
 
-    for (i0 = (i + max_connections - 1) % max_connections; time(NULL) < start + LATENCY && i != i0; i = (i + 1) % max_connections) {
+    for (i0 = (i + max_connections - 1) % max_connections; i != i0; i = (i + 1) % max_connections) {
         connection c;
         int r;
 
         if (!(c = connections[i]))
             continue;
 
+        /* Don't spend too long in this loop. */
+        if (time(NULL) >= start + LATENCY)
+            break;
+
+        if (i > 0 && post_fork) {
+            connections[0] = c;
+            connections[i] = NULL;
+        }
+
         /* Handle all post-select I/O. */
-        r = c->io->post_select(c, readfds, writefds, exceptfds);
+        r = c->io->post_select(c, pfds);
 
-        /* At this stage, the connection may be closed or closing. But we
-         * should try to interpret commands anyway, in case the client sends
-         * QUIT and immediately closes the connection. */
         if (r && !connection_isfrozen(c)) {
             /*
              * Handling of POP3 commands, and forking children to handle
@@ -426,6 +460,15 @@ static void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *e
                     break;
             }
 
+            if (post_fork) {
+                if (i != 0) {
+                    connections[0] = connections[i];
+                    connections[i] = NULL;
+                }
+                i = 0;
+                break;
+            }
+
             if (!c)
                 continue; /* if connection has been destroyed, do next one */
         }
@@ -462,9 +505,19 @@ static void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *e
         if (c->cstate == closed) {
             /* We should now log the closure of the connection and ending
              * of any authenticated session. */
-            if (c->a)
+            if (c->a) {
+                /* Microsoft Outlook closes connections immediately after
+                 * issuing QUIT. By default we'd lose any message deletions
+                 * that were pending, so add an option to apply them even
+                 * so. */
+                if (!config_get_bool("no-commit-on-early-close")) {
+                    pop3command p;
+                    if ((p = connection_parsecommand(c)) && p->cmd == QUIT)
+                        c->m->apply_changes(c->m);
+                }
                 log_print(LOG_INFO, _("connections_post_select: client %s: finished session for `%s' with %s"), c->idstr, c->a->user, c->a->auth);
-            log_print(LOG_INFO, _("connections_post_select: client %s: disconnected; %d/%d bytes read/written"), c->idstr, c->nrd, c->nwr);
+            }
+            log_print(LOG_NOTICE, _("connections_post_select: client %s: disconnected; %d/%d bytes read/written"), c->idstr, c->nrd, c->nwr);
 
 /*            remove_connection(c);*/
             connections[i] = NULL;
@@ -473,6 +526,11 @@ static void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *e
             if (post_fork)
                 _exit(0);
         }
+
+        if (post_fork) {
+            i = 0;
+            break;
+        }
     }
 }
 
@@ -490,6 +548,7 @@ void net_loop(void) {
     extern pid_t child_died;
     extern int child_died_signal;
     sigset_t chmask;
+    struct pollfd *pfds;
     
     sigemptyset(&chmask);
     sigaddset(&chmask, SIGCHLD);
@@ -498,30 +557,32 @@ void net_loop(void) {
     max_connections = 2 * max_running_children;
     connections = (connection*)xcalloc(max_connections, sizeof(connection*));
 
+    pfds = xmalloc(max_connections * sizeof *pfds);
+
     log_print(LOG_INFO, _("net_loop: tpop3d version %s successfully started"), TPOP3D_VERSION);
     
     /* Main select() loop */
     while (!foad) {
-        fd_set readfds, writefds;
-        struct timeval tv = {1, 0}; /* Must be less than IDLE_TIMEOUT and small enough that termination on receipt of SIGTERM is timely. */
-        int n = 0, e;
+        int n = 0, e, i;
 
-        FD_ZERO(&readfds);
-        FD_ZERO(&writefds);
+        for (i = 0; i < max_connections; ++i) {
+            pfds[i].fd = -1;
+            pfds[i].events = pfds[i].revents = 0;
+        }
 
-        if (!post_fork) listeners_pre_select(&n, &readfds, &writefds, NULL);
+        if (!post_fork) listeners_pre_select(&n, pfds);
 
-        connections_pre_select(&n, &readfds, &writefds, NULL);
+        connections_pre_select(&n, pfds);
 
-        e = select(n + 1, &readfds, &writefds, NULL, &tv);
+        e = poll(pfds, n + 1, 1000 /* must be smaller than timeout */);
         if (e == -1 && errno != EINTR) {
-            log_print(LOG_WARNING, "net_loop: select: %m");
+            log_print(LOG_WARNING, "net_loop: poll: %m");
         } else if (e >= 0) {
             /* Check for new incoming connections */
-            if (!post_fork) listeners_post_select(&readfds, &writefds, NULL);
+            if (!post_fork) listeners_post_select(pfds);
 
             /* Monitor existing connections */
-            connections_post_select(&readfds, &writefds, NULL);
+            connections_post_select(pfds);
         }
 
         sigprocmask(SIG_BLOCK, &chmask, NULL);
@@ -556,5 +617,7 @@ void net_loop(void) {
             if (*J) connection_delete(*J);
         xfree(connections);
     }
+
+    xfree(pfds);
 }
 
